import CV_Robot.opencv_api as cv_api
import cv2

CV_BASIC_MODEL = '/some/model/file'

net, classes, output_layers = cv_api.load_model()
camera = cv2.VideoCapture()
camera_active = False

class Objects:
    STOP_SIGN = 'STOP_SIGN'
    BIKE = 'BIKE'
    CAR = 'CAR'
    TRAFFIC_LIGHT = "TRAFFIC_LIGHT"
    FIRE_HYDRANT = "FIRE_HYDRANT"
    PERSON = "PERSON"

def activate_camera():
    global camera
    global camera_active
    camera = cv2.VideoCapture(0, cv2.CAP_DSHOW)
    camera_active = True

def get_camera_image():
    """
    Retrieves current image from robot camera or video
    """
    global camera
    global camera_active
    if not camera_active:
        raise Exception("Camera is not active. Call vision.activate_camera() or vision.load_video()")
    _, img = camera.read()
    return img


def load_image(img_path):
    """
    Loads an image file
    """
    # image loading
    with open(img_path):
        pass #Make sure image exists
    img = cv2.imread(img_path)
    img = cv2.resize(img, None, fx=0.4, fy=0.4)
    return img

def load_video(video_path):
    """
    Loads a video file to emulate camera
    """
    global camera
    global camera_active
    with open(video_path):
        pass #Make sure video exists
    camera = cv2.VideoCapture(video_path)
    camera_active = True

def _map_objects(objs):
    d = {"stop sign": Objects.STOP_SIGN,
         "bicycle": Objects.BIKE,
         "car": Objects.CAR,
         "traffic light": Objects.TRAFFIC_LIGHT,
         "fire hydrant": Objects.FIRE_HYDRANT,
         "person": Objects.PERSON}
    return [d[obj] for obj in objs if obj in d]

def _get_objects(image, thresh=0.3):
    height, width, channels = image.shape
    blob, outputs = cv_api.detect_objects(image, net, output_layers)
    boxes, confs, class_ids = cv_api.get_box_dimensions(outputs, height, width, thresh=thresh)
    return boxes, confs, class_ids

def show_objects(image, thresh=0.3, local_loop=False):
    """
    Displays image with boxes around objects, if block=True, waits for escape to be pressed before closing image
    :param image: Image object (from load_image)
    :param thresh: Threshold to identify object, default is 30% (0.3)
    :param local_loop: Set to true if running in loop outside of colab
    """
    boxes, confs, class_ids = _get_objects(image)
    cv_api.draw_labels(boxes, confs, class_ids, classes, image, thresh=thresh, loop=local_loop)

def find_objects(image, thresh=0.3):
    """
    Runs machine learning model on image specified, returns list of identified objects
    :param image: Image object (from load_image)
    :param thresh: Threshold to identify object, default is 30% (0.3)
    """
    _, _, class_ids = _get_objects(image, thresh=thresh)
    return list(set(_map_objects([classes[x] for x in class_ids])))




try:
    import gpio_cmds
    print("Running on robot...")
    emulate = False
except ImportError:
    print("Running in emulation mode...")
    emulate = True

target_drive_speed = 75

def forward():
    """
    Moves robot forward at speed set by speed command.

    Runs emulated code if GPIO backend not found.
    """
    print("Driving forward...")

def backward():
    """
    Moves robot backward at speed set by speed command.

    Runs emulated code if GPIO backend not found.
    """
    print("Driving backward...")

def left():
    """
    Turns robot left at speed set by speed command.

    Runs emulated code if GPIO backend not found.
    """
    print("Turning left...")

def right():
    """
    Turns robot right at speed set by speed command.

    Runs emulated code if GPIO backend not found.
    """
    print("Turning right...")

def stop():
    """
    Sets speed of all motors to 0.

    Runs emulated code if GPIO backend not found.
    """
    print("Stopping robot...")

def speed(val):
    """
    Sets driving speed to value specified
    """
    global target_drive_speed
    target_drive_speed = val


import matplotlib.pyplot as plt
import numpy as np
import cv2
import os

def load_model():
    """
    Loads Yolo v3 model (called automtically on import)
    """
    yolov3_path = os.path.dirname(__file__) + '/Models/Yolo v3/'
    try:
        with open(yolov3_path + 'yolov3.weights'):
            pass
    except FileNotFoundError:
        print("Downloading model...")
        import wget
        import sys
        def bar_custom(current, total, width=80):
            sys.stdout.write("\r")
            sys.stdout.write("Downloading: %d%% [%d / %d] bytes" % (current / total * 100, current, total))

        wget.download('https://pjreddie.com/media/files/yolov3.weights', yolov3_path + 'yolov3.weights', bar=bar_custom)
        print()

    net = cv2.dnn.readNet(yolov3_path + "yolov3.weights", yolov3_path + "yolov3.cfg")
    with open(yolov3_path + "yolov3.names", "r") as f:
        classes = [line.strip() for line in f.readlines()]

    output_layers = [layer_name for layer_name in net.getUnconnectedOutLayersNames()]
    return net, classes, output_layers



def detect_objects(img, net, outputLayers):
    blob = cv2.dnn.blobFromImage(img, scalefactor=0.00392, size=(320, 320), mean=(0, 0, 0), swapRB=True, crop=False)
    net.setInput(blob)
    outputs = net.forward(outputLayers)
    return blob, outputs

def get_box_dimensions(outputs, height, width, thresh=0.3):
    boxes = []
    confs = []
    class_ids = []
    for output in outputs:
        for detect in output:
            scores = detect[5:]
            class_id = np.argmax(scores)
            conf = scores[class_id]
            if conf > thresh:
                center_x = int(detect[0] * width)
                center_y = int(detect[1] * height)
                w = int(detect[2] * width)
                h = int(detect[3] * height)
                x = int(center_x - w / 2)
                y = int(center_y - h / 2)
                boxes.append([x, y, w, h])
                confs.append(float(conf))
                class_ids.append(class_id)
    return boxes, confs, class_ids

def draw_labels(boxes, confs, class_ids, classes, img, thresh=0.3, loop=False):
    indexes = cv2.dnn.NMSBoxes(boxes, confs, thresh, thresh)
    font = cv2.FONT_HERSHEY_PLAIN
    for i in range(len(boxes)):
        if i in indexes:
            x, y, w, h = boxes[i]
            label = str(classes[class_ids[i]])
            color = (0,0,255) #red
            cv2.rectangle(img, (x, y), (x + w, y + h), color, 2)
            cv2.putText(img, label, (x, y - 5), font, 1, color, 1)

    plt.imshow(img)
    plt.axis('off')

    if loop:
        plt.draw()
        plt.pause(0.1)
    else:
        plt.show()



#This is a barebones setup.py file, because we use setup.cfg instead!
#setup.cfg is declarative, and therefore better for security reasons,
from setuptools import setup

if __name__ == "__main__":
    setup()